/* 
DE1K Variables 
	You can change the values of these color and font-size variables to customize your experience.
*/

:root {
  /* Increase this max-width-card value to increase the maximum width of your card! */
  --max-width-card: 400px;

  /* Increase the font-size to make the card easier to read! If you want to make the text bigger and easier to see, we suggest that you also increase the max-width of the card so you have more space. The card's max-width variable is the first variable, above this comment. */ 
  --font-size-card: 18px;
  --font-size-targetWord: 26px;
  --font-size-header: 16px;
  --font-size-irregular-form: 18px;

  /* Change the color values of these variables to change the colors in your cards! NightMode colors are only active when you are running anki in Night Mode (dark mode). */ 
  --color-text-primary: #18191f;
  --color-nightMode-text-primary: #fbfafe;
  --color-card-background: #ffffff;
  --color-nightMode-card-background: #0b0716;
  --color-box-shadow: rgba(18, 62, 119, 0.1);
  --color-audio-button: #8369ed;
  --color-list-bullets: #8369ed;
  --color-irregular-form-background: #f3f4f6;
  --color-nightMode-irregular-form-background: #1f2937;
  --color-irregular-form-border: #d1d5db;
  --color-nightMode-irregular-form-border: #4b5563;
  --color-nightMode-irregular-form-text: rgba(255, 255, 255, 0.7);
  --color-hint: #6b7280;
  --color-nightMode-hint: #9ca3af;
  --color-sentence-translation: #6b7280;
  --color-nightMode-sentence-translation: #9ca3af;
  --color-header: #9ca3af;
  --color-nightMode-header: rgba(255, 255, 255, 0.5);
  --color-divider: #e5e7eb;
  --color-nightMode-divider: #1f2937;
}

/* Chrome Base Styles */
p {
  display: block;
  margin-block-start: 1em;
  margin-block-end: 1em;
  margin-inline-start: 0px;
  margin-inline-end: 0px;
}

/* Anki Browser Styles */
div {
  display: block;
}

body {
  display: block;
  line-height: 1.187;
}

html {
  display: block;
  color:  -internal-root-color;
}

/* http://meyerweb.com/eric/tools/css/reset/
   v2.0 | 20110126
   License: none (public domain)
*/

span, applet, object, iframe,
h1, h2, h3, h4, h5, h6, blockquote, pre,
a, abbr, acronym, address, big, cite, code,
del, dfn, em, img, ins, kbd, q, s, samp,
small, strike, strong, sub, sup, tt, var,
b, u, i, center,
dl, dt, dd, ol, ul, li,
fieldset, form, label, legend,
table, caption, tbody, tfoot, thead, tr, th, td,
article, aside, canvas, details, embed,
figure, figcaption, footer, header, hgroup,
menu, nav, output, ruby, section, summary,
time, mark, audio, video {
  margin: 0;
  padding: 0;
  border: 0;
  font-size: 100%;
  font: inherit;
  vertical-align: baseline;
}
/* HTML5 display-role reset for older browsers */
article, aside, details, figcaption, figure,
footer, header, hgroup, menu, nav, section {
  display: block;
}

ol, ul {
  list-style: none;
}
blockquote, q {
  quotes: none;
}
blockquote:before, blockquote:after,
q:before, q:after {
  content: '';
  content: none;
}
table {
  border-collapse: collapse;
  border-spacing: 0;
}

/* Refold Specific Styles */

/* Anki 2.1.49 sets box-sizing to content-box  */
* {
  box-sizing: border-box;
}

/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

body.nightMode::-webkit-scrollbar {
  background: #2f2f31;
}

body.nightMode::-webkit-scrollbar:horizontal {
  height: 12px;
}

body.nightMode::-webkit-scrollbar:vertical {
  width: 12px;
}

body.nightMode::-webkit-scrollbar-thumb {
  background: #656565;
  border-radius: 8px;
}

body.nightMode::-webkit-scrollbar-thumb:horizontal {
  min-width: 50px;
}

body.nightMode::-webkit-scrollbar-thumb:vertical {
  min-height: 50px;
}

body {
  overscroll-behavior: none;
}

/* night-mode-specific colours */
/* Copyright: Ankitects Pty Ltd and contributors
 * License: GNU AGPL, version 3 or later; http://www.gnu.org/licenses/agpl.html */

body {
  margin: 0 !important;
  overflow-wrap: break-word;
}

#_flag {
  position: fixed;
  right: 10px;
  top: 0;
  font-size: 30px;
  display: none;
  -webkit-text-stroke-width: 1px;
  -webkit-text-stroke-color: black;
}

#_mark {
  position: fixed;
  left: 10px;
  top: 0;
  font-size: 30px;
  color: yellow;
  display: none;
  -webkit-text-stroke-width: 1px;
  -webkit-text-stroke-color: black;
}

#typeans {
  width: 100%;
  box-sizing: border-box;
}

.typeGood {
  background: #0f0;
}

.typeBad {
  background: #f00;
}

.typeMissed {
  background: #ccc;
}

.nightMode .latex {
  filter: invert(100%);
}

.drawing {
  zoom: 50%;
}

.nightMode img.drawing {
  filter: invert(1) hue-rotate(180deg);
}


/* DE1K STYLES */

.card {
  padding: 16px;
}

.customCard {
  margin: 0 auto;

  position: relative;
  display: flex;
  justify-content: center;
  align-items: center;

  background-color: var(--color-card-background);
  box-shadow: 1px 3px 10px var(--color-box-shadow);
  border-radius: 8px;
  min-height: 200px;
  max-width: var(--max-width-card);

  font-weight: 500;
  font-size: var(--font-size-card);
  font-family: "Inter", sans-serif;
  color: var(--color-text-primary);
}

.nightMode .customCard {
  color: var(--color-nightMode-text-primary);
  background-color: var(--color-nightMode-card-background);
}

.cardBack {
  flex-direction: column;
  justify-content: flex-start;
  align-items: flex-start;

  padding-top: 24px;
  padding-bottom: 8px;
}

.targetWordContainer {
  margin-top: 8px;
  margin-bottom: 24px;
}

.targetWord {
  font-size: var(--font-size-targetWord);
  font-weight: 600;
}

.definitionsList {
  list-style: none;
  margin: 0;
  padding: 0;
  padding-left: 16px;
}

.definitionsList li::before {
  content: "\2022";
  color: var(--color-list-bullets);
  display: inline-block;
  width: 8px;
  margin-left: -16px;
  font-size: 16px;
}

.wordAudioButtonFront {
  position: absolute;
  top: 21px;
  right: 22px;
}
.wordAudioButtonBack {
  margin-left: 8px;
}
.replay-button svg {
  width: 20px;
  height: 20px;
}
.replay-button svg path {
  fill: var(--color-audio-button);
}
.replay-button svg circle {
  fill: none;
  stroke: none;
}

.irregularFormsContainer {
  display: flex;
  flex-wrap: wrap;

  margin-top: 12px;
}

.irregularForm {
  background-color: var(--color-irregular-form-background);
  border: 0.5px solid var(--color-irregular-form-border);
  border-radius: 40px;

  font-size: var(--font-size-irregular-form);

  padding: 3px 12px 5px;
  margin-right: 10px;
  margin-bottom: 8px;
}

.nightMode .irregularForm {
  color: var(--color-nightMode-irregular-form-text);
  background-color: var(--color-nightMode-irregular-form-background);
  border-color: var(--color-nightMode-irregular-form-border);
}

.image {
  width: 100%;
  height: 210px;
  margin-top: 8px;
  margin-bottom: 24px;
}

.image img {
  object-fit: contain;
  width: 100% !important;
  height: 100% !important;
  max-width: 100% !important;
  max-height: 100% !important;
}

.hint {
  color: var(--color-hint);
}

.nightMode .hint {
  color: var(--color-nightMode-hint);
}

.sentenceTranslation {
  color: var(--color-sentence-translation);
  font-weight: 400;
  font-style: italic;
  padding-right: 15px;
  padding-top: 8px;
  font-size: var(--font-size-header);
}

.nightMode .sentenceTranslation {
  color: var(--color-nightMode-sentence-translation);
}

.reportIconContainer {
  width: 100%;
  display: flex;
  justify-content: flex-end;
  align-items: center;
}

.header {
  color: var(--color-header);
  font-size: var(--font-size-header);
  font-weight: 400;
  margin-top: 8px;
  margin-bottom: 8px;
}

.nightMode .header {
  color: var(--color-nightMode-header);
}

.dividerLine {
  width: 100%;
  border-bottom: 1px solid var(--color-divider);
  margin-top: 8px;
  margin-bottom: 8px;
}

.nightMode .dividerLine {
  border-color: var(--color-nightMode-divider);
}

.horizontalPadding {
	width: 100%;
  padding-left: 16px;
  padding-right: 16px;
}

.indent {
  padding-left: 12px;
}

.centerVertically {
  display: flex;
  align-items: center;
}

@font-face {
  font-family: Inter;
  src: url("_Inter-Regular.ttf");
  font-weight: 400;
}
@font-face {
  font-family: Inter;
  src: url("_Inter-Medium.ttf");
  font-weight: 500;
}
@font-face {
  font-family: Inter;
  src: url("_Inter-SemiBold.ttf");
  font-weight: 600;
}

.speaker {
  color: var(--color-sentence-translation);
  font-weight: 400;
}